*Program file of project fdz371 - p2_summarize.do
*Contents: creates data sets for different parts of the analysis and provides summary statistics

*The datasets created and summarized are:
*1. p2_summarize_wage.dta: contains the variables needed for the main regression, i.e. the match fixed effects model. 
*   Level of aggregation: none, unit of observation is the individual/year
*2. p2_sumamrize_matlab.txt: Same as the above, but with variables coded to make matlab program work (dummies expanded, higher order terms created,...)
*   Level of aggregation: none, unit of observation is the individual/year
*3. p2_summarize_firm.dta: contains the variables used as covariates to explain the firm fixed effects. 
*   Level of aggregation: firm, i.e. unit of observation is the firm. Data is aggregated over time by taking the modal value for each firm.
*4. p2_summarize_indiv.dta: contains the variables used as covariates to explain the individual fixed effects.
*   Level of aggregation: individual, i.e. unit of observation is the individual. Data is aggregated over time by taking the modal value for each individual
*5. p2_summarize_match.dta: contains the variables used as covariates to explain the match fixed effects.
*   Level of aggregation: match, i.e. unit of observation are unique individual-firm combinations. Data is aggregated over time by taking the value at the beginning of each combination.
*Details of the statistics created and the number of observations they are based on are provided below.

*Additional files created: p2_summarize_firm.log p2_summarize_indiv.log, p2_summarize_match.log, p2_summarize_wage.log (log files)

*Level of aggregation: see individual files above


log using log/p2_summarize_firm.log, replace text
*create dataset with covariates to regress firm fixed effects on: this aggregates some variables at the firm level. The resulting data set 
*contains one observation per firm with the most frequent value of the original variable. 
preserve
*keep only required variables
keep idnum hrf_quer bran bula form eigentum grjahr single weiterb betrrat tarif work_own jahr

*collapse data to firm level
duplicates drop idnum jahr, force

*adjust weights so that each year gets same weight
qui: sum jahr
local anf=r(min)
local ende=r(max)
foreach x of numlist `anf'/`ende' {
qui: total hrf_quer if jahr==`x'
mat def a=e(b)
local a=a[1,1]
replace hrf_quer=hrf_quer/`a' if jahr==`x'
}

*make variables constant within firms
sort idnum
by idnum: egen bran_c=mode(bran), maxmode
by idnum: egen bula_c=mode(bula), maxmode
by idnum: egen form_c=mode(form), minmode
by idnum: egen eigentum_c=mode(eigentum), minmode
by idnum: egen grjahr_c=mode(grjahr), minmode
by idnum: egen single_c=mode(single), minmode
by idnum: egen weiterb_c=mode(weiterb), maxmode
by idnum: egen betrrat_c=mode(betrrat), maxmode
by idnum: egen tarif_c=mode(tarif), minmode
by idnum: egen work_own_c=mode(work_own), maxmode

*get cross tabulations of edited & unedited variable (to make sure there are not "too many" off-diagonal elements)
foreach x in bran bula form eigentum grjahr single weiterb betrrat tarif work_own {
tab `x' `x'_c, mis
}

*rename variables and store labels
foreach x in bran bula form eigentum grjahr single weiterb betrrat tarif work_own {
local l`x' : variable label `x'
if `"`l`x''"' == "" {
local l`x' "`x'"
}
replace `x'=`x'_c
}

*drop unedited and other variables
keep idnum hrf_quer bran bula form eigentum grjahr single weiterb betrrat tarif work_own

*take inverse of weight
replace hrf_quer=1/hrf_quer

*collapse to firm level
collapse (firstnm) bran bula form eigentum grjahr single weiterb betrrat tarif work_own (mean) hrf_quer, by(idnum)

*re-attach labels
foreach x of var bran bula form eigentum grjahr single weiterb betrrat tarif work_own {
label var `x' "`l`x''"
}
foreach x in bran bula form eigentum grjahr single tarif {
label val `x' `x'lb
}
label val work_own dummy

*take inverse of weight again
replace hrf_quer=1/hrf_quer
compress

*get frequencies for categorical variables: the following tables give the weighted frequencies for each variable.
*For each variable, there also is a table with unweighted frequencies that give the number of observations on which
*each number is calculated. Variables are aggregated at the firm level, i.e. an observation combines information
*on the same firm for all years.
*note: should use pweight, but aweight gives the same result in this case
foreach x in bran bula form eigentum grjahr single weiterb betrrat tarif work_own {
tab `x' [aw=hrf_quer], missing
tab `x', missing
}

*get summary statistics
tabstat bran bula form eigentum grjahr single weiterb betrrat tarif work_own [aw=hrf_quer], stat(mean sd min max n) c(statistics)

*recode missing values to include missing dummy
recode bran bula form eigentum grjahr single weiterb betrrat tarif work_own (.=9999)

describe
save data/p2_summarize_firm.dta, replace
clear
capture log close
restore

log using log/p2_summarize_indiv.log, text replace


*generate match id
egen matchid=group(persnr idnum)

*adjust weights for # of people per firm
*number of people per firm in sample
bysort idnum jahr: egen nfirm=count(persnr)
*identify movers
egen tmp=tag(persnr idnum)
bysort persnr: egen wadjust=total(tmp)
replace wadjust=wadjust>1
bysort idnum jahr: egen nmover=total(wadjust)
*adjust weights
replace wadjust=max(1,(nfirm-nmover)/(az_ges-nmover)) if wadjust==0
*summarize adjustment to make sure nothing goes wrong
sum wadjust
replace hrf_quer=hrf_quer*wadjust
drop nfirm wadjust az_ges nmover
*adjust weights so that each year gets same weight
foreach x of numlist `anf'/`ende' {
qui: total hrf_quer if jahr==`x'
mat def a=e(b)
local a=a[1,1]
replace hrf_quer=hrf_quer/`a' if jahr==`x'
}
sum hrf_quer


*label variables
label var jahr "Befragungsjahr"
label var age "Alter am Ende des Jahres"
label var pot_exp "Alter-Ausbildung-6"
label var parttime "Teilzeitarbeit"
label var days_lm "Tage seit Eintritt Erwerbsleben bei Arbeitsbeginn"
label var topcode "ltag_entg topcoded"
label var matchid "Match-ID"
label var ltag_entg "ln Tagesentgelt/Leistungssatz"
label var yrentry "Eintrittsjahr Arbeitsmarkt"
label var agentry "Alter bei Eintritt in den Arbeitsmarkt"

label val topcode parttime dummy


*examine relationship between potential and actual experience by gender
bysort frau: pwcorr pot_exp tage_erw
bysort frau: sum pot_exp
bysort frau: sum tage_erw
xi: reg tage_erw i.frau*pot_exp

*create dataset with covariates to regress individual fixed effects on: this aggregates some variables at the individual level. The resulting data set 
*contains one observation per individual with the most frequent value of the original variable. 
sort persnr jahr
by persnr: egen bild_c=mode(bild), maxmode
*get cross tabulation of topcoded observations by education to see whether and if, how, I could get a sample with few topcoded people by constraining education
***get rid of this in final program
tab bild topcode

*generate variable to restrict sample based on education
gen rsample=1
replace rsample=0 if bild==5 | bild==6 | bild_c==5 | bild_c==6
label var rsample "In reduced Sample"
label val rsample dummy
tab topcode rsample
tab bild topcode if rsample==1
preserve
keep persnr hrf_quer frau nation_gr bild bild_c yrentry agentry rsample
by persnr: egen frau_c=mode(frau), maxmode
by persnr: egen nation_gr_c=mode(nation_gr), maxmode
by persnr: egen yrentry_c=mode(yrentry), minmode
by persnr: egen agentry_c=mode(agentry), minmode

*get cross tabulations of edited & unedited variable (to make sure there are not "too many" off-diagonal elements), replace unedited by edited variables, store labels
foreach x in frau nation_gr bild yrentry agentry {
tab `x' `x'_c, mis
local l`x' : variable label `x'
if `"`l`x''"' == "" {
local l`x' "`x'"
}
replace `x'=`x'_c
drop `x'_c
}

*take inverse of weights
replace hrf_quer=1/hrf_quer

*collapse
collapse (firstnm) frau nation_gr bild yrentry agentry (mean) hrf_quer (max) rsample, by(persnr)

*re-attach labels
foreach x in frau nation_gr bild yrentry agentry {
label var `x' "`l`x''"
}
label val frau frau_en
label val nation_gr nation_gr_en
label val bild bild_en

*take inverse of weights again
replace hrf_quer=1/hrf_quer
compress

*get frequencies for categorical variables: the following tables give the frequencies for each variable. There are 3 tables
*each variable: the first two use the two weights, the third uses no weights. Variables are aggregated at the individual level, 
*i.e. an observation combines information on the same individual for all years.
*note: should use pweight, but aweight gives the same result in this case
foreach x in frau nation_gr bild yrentry {
disp "weighted frequencies, full sample"
tab `x' [aw=hrf_quer], missing
disp "unweighted frequencies, full sample"
tab `x', missing
disp "weighted frequencies, reduced sample"
tab `x' if rsample==1 [aw=hrf_quer], missing
disp "unweighted frequencies, reduced sample"
tab `x' if rsample==1, missing
}

*Get summary statistics for year and age when entering labor market. They are based on one observation per individual, the last three columns give min, max and the 
*number of observations used to calculate each line. The first table uses the weights for the full sample, the second one for the restricted sample.
*note: should use pweight, but aweight gives the same result in this case
tabstat yrentry agentry frau nation_gr bild [aw=hrf_quer], stat(mean sd min max n) c(statistics)
tabstat yrentry agentry frau nation_gr bild if rsample==1 [aw=hrf_quer], stat(mean sd min max n) c(statistics)

*recode missing values to include missing dummy
recode frau nation_gr bild yrentry agentry (.=9999)

save data/p2_summarize_indiv.dta, replace
describe
clear
capture log close
restore
drop bild bild_c
log using log/p2_summarize_match.log, text replace

*create dataset with covariates to regress match/firm fixed effects on: this aggregates some variables at the match (i.e. unique person/firm combination) level. The resulting data set 
*contains one observation per match which takes the value of the orignial variable at the beginning of the match
preserve
keep idnum persnr matchid hrf_quer jahr days_lm tage_lst2 age parttime frau tage_estat2_8tv estat2_8tv rsample

*take inverse of weight
replace hrf_quer=1/hrf_quer

foreach x in idnum persnr jahr days_lm tage_lst2 age parttime frau tage_estat2_8tv estat2_8tv {
local l`x' : variable label `x'
if `"`l`x''"' == "" {
local l`x' "`x'"
}
}

sort persnr jahr
collapse (firstnm) idnum persnr jahr days_lm tage_lst2 age parttime frau tage_estat2_8tv estat2_8tv (mean) hrf_quer (max) rsample, by(matchid)

*re-attach labels
foreach x in idnum persnr jahr days_lm tage_lst2 age parttime frau tage_estat2_8tv estat2_8tv {
label var `x' "`l`x''"
}
label val parttime dummy
label val frau frau_en

*take inverse of weight again
replace hrf_quer=1/hrf_quer

*create match count
bysort persnr: egen n_match=seq()
label var n_match "Match Nummer"

*get frequencies for categorical variables: the following tables give the frequencies for each variable. There are 3 tables
*each variable: the first two use the two weights, the third uses no weights. Variables are aggregated at the match level, 
*i.e. an observation combines information on the same individual/firm combination for all years.
*note: should use pweight, but aweight gives the same result in this case
foreach x in parttime estat2_8tv frau jahr n_match {
disp "weighted frequencies, full sample"
tab `x' [aw=hrf_quer], missing
disp "unweighted frequencies, full sample"
tab `x', missing
disp "weighted frequencies, reduced sample"
tab `x' if rsample==1 [aw=hrf_quer], missing
disp "unweighted frequencies, reduced sample"
tab `x' if rsample==1, missing
}

*same tabulations as previous by gender for number of matches.
disp "weighted frequencies, full sample"
bysort frau: tab n_match [aw=hrf_quer], missing
disp "unweighted frequencies, full sample"
bysort frau: tab n_match, missing
disp "weighted frequencies, reduced sample"
bysort frau: tab n_match if rsample==1 [aw=hrf_quer], missing
disp "unweighted frequencies, reduced sample"
bysort frau: tab n_match if rsample==1, missing


*generate summary statistics for other variables. They are based on one observation per match, the last three columns give min, max and the 
*number of observations used to calculate each line. The first table uses the weights for the full sample, the second one for the restricted sample.
*note: should use pweight, but aweight gives the same result in this case
tabstat matchid days_lm tage_lst2 tage_estat2_8tv n_match age parttime estat2_8tv frau jahr [aw=hrf_quer], stat(mean sd min max n) c(statistics)
tabstat matchid days_lm tage_lst2 tage_estat2_8tv n_match age parttime estat2_8tv frau jahr if rsample==1 [aw=hrf_quer], stat(mean sd min max n) c(statistics)

*get summary statistics by gender for some key covariates
bysort frau: sum age days_lm tage_lst2 parttime [aw=hrf_quer]
bysort frau: sum age days_lm tage_lst2 parttime if rsample==1 [aw=hrf_quer]

*get counts of missing values for continuous variables
foreach x in days_lm tage_lst2 tage_estat2_8tv n_match {
tab `x' if `x'==., mis
}

*recode missing values to include missing dummy
recode parttime estat2_8tv frau jahr (.=9999)

compress
sort matchid
describe
save data/p2_summarize_match.dta, replace
clear
capture log close
restore

log using log/p2_summarize_wage.log, text replace
*keep only variables for main regression
keep persnr idnum matchid ltag_entg jahr hrf_quer gesamt d_geschvol d_rev_pw frac_pt d_inv_pw frac_fem DHSindex emp_grd fhire d_grow einstell hiring entlass firing tage_bet age parttime tage_erw pot_exp topcode rsample


*get frequencies for categorical variables: the following tables give the weighted frequencies for each variable.
*There are 3 tables for each variable: the first two use the two weights, the third uses no weights. Variables are not aggregated.
*note: should use pweight, but aweight gives the same result in this case
foreach x in jahr fhire d_grow emp_grd hiring firing parttime topcode rsample d_geschvol d_rev_pw d_inv_pw {
disp "weighted frequencies, full sample"
tab `x' [aw=hrf_quer], missing
disp "unweighted frequencies, full sample"
tab `x', missing
disp "weighted frequencies, reduced sample"
tab `x' if rsample==1 [aw=hrf_quer], missing
disp "unweighted frequencies, reduced sample"
tab `x' if rsample==1, missing
}

*generate summary statistics for other variables. Variables are not aggregated, the last three columns give min, max and the 
*number of observations used to calculate each line. The first table uses the weights for the full sample, the second one for the restricted sample.
*note: should use pweight, but aweight gives the same result in this case
tabstat ltag_entg gesamt frac_pt frac_fem DHSindex einstell entlass tage_bet age tage_erw pot_exp jahr fhire d_grow emp_grd hiring firing parttime topcode rsample d_geschvol d_rev_pw d_inv_pw [aw=hrf_quer], stat(mean sd min max n) c(statistics)
tabstat ltag_entg gesamt frac_pt frac_fem DHSindex einstell entlass tage_bet age tage_erw pot_exp jahr fhire d_grow emp_grd hiring firing parttime topcode rsample d_geschvol d_rev_pw d_inv_pw if rsample==1 [aw=hrf_quer], stat(mean sd min max n) c(statistics)

*get number of individuals, firms and matches
*overall number of matches, firms, individuals
codebook matchid persnr idnum
*# of individuals and firms by year
egen tmp=tag(idnum jahr)
table jahr, contents(n persnr sum tmp)
table jahr if rsample==1, contents(n persnr sum tmp)
drop tmp

compress
save data/p2_summarize_wage.dta, replace

*create file for Matlab

*create dummies for categorical variables
foreach x in jahr fhire d_grow hiring firing parttime d_geschvol d_inv_pw {
qui: tab `x', missing gen(`x'_d)
drop `x'_d1 `x'
}

*dummies for categorical vars where missing category needs to be excluded b/c its perfectly colinear
qui: tab d_rev_pw, gen(d_rev_pw_d)
drop d_rev_pw_d1 d_rev_pw d_rev_pw_d11


*create higher order terms
foreach x in age tage_erw pot_exp {
gen `x'_sq=`x'^2
gen `x'_cu=`x'^3
gen `x'_qu=`x'^4
}

*get correlation matrix of variables
pwcorr ltag_entg gesamt d_geschvol* d_rev_pw* frac_pt d_inv_pw* frac_fem DHSindex emp_grd* fhire* d_grow* einstell hiring* entlass firing* tage_bet age* parttime* topcode jahr* tage_erw* pot_exp* 


drop hrf_quer age pot_exp
order persnr idnum matchid ltag_entg gesamt d_geschvol* d_rev_pw* frac_pt d_inv_pw* frac_fem DHSindex emp_grd* fhire* d_grow* einstell hiring* entlass firing* tage_bet age* parttime* topcode jahr* tage_erw* pot_exp* rsample

*make sure there are no missing values in final matrix
tabstat _all, stat(n) c(statistics)
compress
describe
*save number of observations and variables for Matlab to read
file open info using data/p2_summarize_info.txt, write replace
file write info %10.0g (r(N)) _tab %4.0g (r(k))
file close info

outsheet _all using data/p2_summarize_matlab.txt, replace nolabel
clear *
set mem 1m
capture log close
