%Program file of project fdz371 - p3_mat.m
%Content: Runs several variants of the match effects and two way fixed effects model on the dataset "p2_summarize_matlab.txt" created by the Stata .do file p2_summarize.do

%Files created: 
%in ./log: a comma separated file containing the coefficients and their covariance matrix for every model estimated, named p3_mat_b_*.csv, where * is a shorthand for the model
%in ./data: comma separated files containing the estimates of the firm fixed effects (p3_mat_ffe_*.csv), individual fixed effects (p3_mat_ife_*.csv) for all models and a file 
%containing the match fixed effects (p3_mat_mfe_*.csv) for the models that include match fixed effects.

%Level of aggregation: the data used in this file is not aggregated. In the output files in ./data, an observation represents a firm (p3_mat_ffe_*.csv) an individual (p3_mat_ife_*.csv) or a 
%unique combination of firm and individual (p3_mat_mfe_*.csv).

disp('Loading Data and Preparing Things')
%include prog folder in matlab path to be able to access fixed function, set display format
addpath(cd)
format long
%import data
cd ..
cd data
%get # of obs and variables
[n,k]=textread('p2_summarize_info.txt','%f %f')
a=repmat('%f ',1,k);
fid=fopen('p2_summarize_matlab.txt');
disp('Starting to load data')
colheaders=textscan(fid,'%s',k);
data=textscan(fid,a,n);
fclose(fid);
clear a k n
data=cell2mat(data);
disp('Dimensions of data matrix to make sure nothing went wrong')
size(data)
disp('Vector of variable names, just to check that it is right')
colheaders{1}

%identify connected groups, keep only largest
g_id=size(data,2)+1;
i_id=find(ismember(colheaders{1},{'persnr'}));
f_id=find(ismember(colheaders{1},{'idnum'}));
i=0;
data(:,g_id)=0;
data=sortrows(data,[i_id f_id]);

while nnz(data(:,g_id))<size(data,1),
    i=i+1;
    b=zeros(max(data(:,i_id)),1);
    d=zeros(max(data(:,f_id)),1);
    g=zeros(size(data,1),1);
    alag=[];
    g(data(:,f_id)==data(find(data(:,g_id)==0,1),f_id))=i;
    a=unique(data(g==i,i_id));

while length(a)>length(alag),
    b(a)=1;
    g=(b(data(:,i_id))==1) | (g==1);
    alag=a;
    c=unique(data(g==1,f_id));
    d(c)=1;
    g=(d(data(:,f_id))==1) | (g==1);
    a=unique(data(g==1,i_id));
end
data(:,g_id)=data(:,g_id)+g.*i;
end
clear a alag b c d g i
disp('Number of observations in 20 largest connected groups')
a=histc(data(:,g_id),1:max(data(:,g_id)));
b=sortrows(a,-1);
b(1:20)
disp('Number of observations in remaining groups')
sum(b(21:end))

%keep only observations in largest group
[a,b]=max(a);
data=data(data(:,g_id)==b,1:end-1);
clear a b

%run match effects model with potential experience
disp('Match effects model with potential experience')
vnames=colheaders{1}(~ismember(colheaders{1},{'persnr' 'idnum' 'matchid' 'ltag_entg' 'tage_erw' 'tage_erw_sq' 'tage_erw_cu' 'tage_erw_qu' 'rsample'}));
tic
[bpotexp,fepotexp,fpotexp]=fixed(data(:,~ismember(colheaders{1},{'tage_erw' 'tage_erw_sq' 'tage_erw_cu' 'tage_erw_qu' 'rsample'})),4,5:4+length(vnames),1:3,'coef',1:2)
toc
disp('Output')
%arrange and save output
bpotexp.beta(:,2)=sqrt(diag(bpotexp.covb));
disp('Writing fixed effects to file, numbers below just indicate whether files were opened (fid=3), how many bytes were written and whether files were closed')
disp('individual fixed effects')
fid=fopen('p3_mat_ife_pem.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe1))
fclose(fid)
disp('firm fixed effects')
fid=fopen('p3_mat_ffe_pem.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe2))
fclose(fid)
disp('match fixed effects')
fid=fopen('p3_mat_mfe_pem.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.me))
fclose(fid)

disp('Regression Output')
disp('Statistics: # of observations, # of individuals, # of firms, # of matches, total sum of squares, residual sum of squares, R2, F-statistics')
fpotexp
disp('Coefficients and SEs')
for j=1:length(vnames),
    fprintf(1,'%s \t %12.8e \t %12.8e \n',char(vnames(j)),bpotexp.beta(j,1),bpotexp.beta(j,2));
end
disp('Covariance Matrix of the coefficient estimates')
bpotexp.covb
disp('below are just some checks whether things went right')
%get number and size of connected groups
disp('Number of people in each connected group')
histc(fepotexp.fe1(:,3),1:max(fepotexp.fe1(:,3)))
disp('Number of firms in each connected group')
histc(fepotexp.fe2(:,3),1:max(fepotexp.fe2(:,3)))
%check to see whether any effects were not calculated
disp('Missing fixed effects')
disp('individual')
sum(isnan(fepotexp.fe1))
disp('firm')
sum(isnan(fepotexp.fe2))
disp('match')
sum(isnan(fepotexp.me))
disp('# of non-zero match effects')
sum(abs(fepotexp.me(:,4))>0.000001)
clear bpotexp fepotexp fpotexp

%run two way fixed effects model with potential experience
disp('Two way fixed effects model with potential experience')
tic
[bpotexp,fepotexp,fpotexp]=fixed(data(:,~ismember(colheaders{1},{'tage_erw' 'tage_erw_sq' 'tage_erw_cu' 'tage_erw_qu' 'rsample'})),4,5:4+length(vnames),1:2,'coef',1:2)
toc
disp('Output')
%arrange and save output
disp('Writing fixed effects to file, numbers below just indicate whether files were opened (fid=3), how many bytes were written and whether files were closed')
disp('individual fixed effects')
bpotexp.beta(:,2)=sqrt(diag(bpotexp.covb));
fid=fopen('p3_mat_ife_pet.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe1))
fclose(fid)
disp('firm fixed effects')
fid=fopen('p3_mat_ffe_pet.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe2))
fclose(fid)

disp('Regression Output')
disp('Statistics: # of observations, # of individuals, # of firms, # of matches, total sum of squares, residual sum of squares, R2, F-statistics')
fpotexp
disp('Coefficients and SEs')
for j=1:length(vnames),
    fprintf(1,'%s \t %12.8e \t %12.8e \n',char(vnames(j)),bpotexp.beta(j,1),bpotexp.beta(j,2));
end
disp('Covariance Matrix of the coefficient estimates')
bpotexp.covb
disp('below are just some checks whether things went right')
%get number and size of connected groups
disp('Number of people in each connected group')
histc(fepotexp.fe1(:,3),1:max(fepotexp.fe1(:,3)))
disp('Number of firms in each connected group')
histc(fepotexp.fe2(:,3),1:max(fepotexp.fe2(:,3)))
%check to see whether any effects were not calculated
disp('Missing fixed effects')
disp('individual')
sum(isnan(fepotexp.fe1))
disp('firm')
sum(isnan(fepotexp.fe2))
clear fepotexp fepotexp fpotexp

%run match effects model with actual experience
disp('Match effects model with actual experience')
vnames=colheaders{1}(~ismember(colheaders{1},{'persnr' 'idnum' 'matchid' 'ltag_entg' 'pot_exp_sq' 'pot_exp_cu' 'pot_exp_qu' 'rsample'}));
tic
[bpotexp,fepotexp,fpotexp]=fixed(data(:,~ismember(colheaders{1},{'pot_exp_sq' 'pot_exp_cu' 'pot_exp_qu' 'rsample'})),4,5:4+length(vnames),1:3,'coef',1:2)
toc
disp('Output')
%arrange and save output
bpotexp.beta(:,2)=sqrt(diag(bpotexp.covb));
disp('Writing fixed effects to file, numbers below just indicate whether files were opened (fid=3), how many bytes were written and whether files were closed')
disp('individual fixed effects')
fid=fopen('p3_mat_ife_am.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe1))
fclose(fid)
disp('firm fixed effects')
fid=fopen('p3_mat_ffe_am.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe2))
fclose(fid)
disp('match fixed effects')
fid=fopen('p3_mat_mfe_am.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.me))
fclose(fid)

disp('Regression Output')
disp('Statistics: # of observations, # of individuals, # of firms, # of matches, total sum of squares, residual sum of squares, R2, F-statistics')
fpotexp
disp('Coefficients and SEs')
for j=1:length(vnames),
    fprintf(1,'%s \t %12.8e \t %12.8e \n',char(vnames(j)),bpotexp.beta(j,1),bpotexp.beta(j,2));
end
disp('Covariance Matrix of the coefficient estimates')
bpotexp.covb
disp('below are just some checks whether things went right')
%get number and size of connected groups
disp('Number of people in each connected group')
histc(fepotexp.fe1(:,3),1:max(fepotexp.fe1(:,3)))
disp('Number of firms in each connected group')
histc(fepotexp.fe2(:,3),1:max(fepotexp.fe2(:,3)))
%check to see whether any effects were not calculated
disp('Missing fixed effects')
disp('individual')
sum(isnan(fepotexp.fe1))
disp('firm')
sum(isnan(fepotexp.fe2))
disp('match')
sum(isnan(fepotexp.me))
disp('# of non-zero match effects')
sum(abs(fepotexp.me(:,4))>0.000001)
clear bpotexp fepotexp fpotexp

%run two way fixed effects model with actual experience
disp('Two way fixed effects model with actual experience')
tic
[bpotexp,fepotexp,fpotexp]=fixed(data(:,~ismember(colheaders{1},{'pot_exp_sq' 'pot_exp_cu' 'pot_exp_qu' 'rsample'})),4,5:4+length(vnames),1:2,'coef',1:2)
toc
disp('Output')
%arrange and save output
disp('Writing fixed effects to file, numbers below just indicate whether files were opened (fid=3), how many bytes were written and whether files were closed')
disp('individual fixed effects')
bpotexp.beta(:,2)=sqrt(diag(bpotexp.covb));
fid=fopen('p3_mat_ife_at.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe1))
fclose(fid)
disp('firm fixed effects')
fid=fopen('p3_mat_ffe_at.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe2))
fclose(fid)

disp('Regression Output')
disp('Statistics: # of observations, # of individuals, # of firms, # of matches, total sum of squares, residual sum of squares, R2, F-statistics')
fpotexp
disp('Coefficients and SEs')
for j=1:length(vnames),
    fprintf(1,'%s \t %12.8e \t %12.8e \n',char(vnames(j)),bpotexp.beta(j,1),bpotexp.beta(j,2));
end
disp('Covariance Matrix of the coefficient estimates')
bpotexp.covb
disp('below are just some checks whether things went right')
%get number and size of connected groups
disp('Number of people in each connected group')
histc(fepotexp.fe1(:,3),1:max(fepotexp.fe1(:,3)))
disp('Number of firms in each connected group')
histc(fepotexp.fe2(:,3),1:max(fepotexp.fe2(:,3)))
%check to see whether any effects were not calculated
disp('Missing fixed effects')
disp('individual')
sum(isnan(fepotexp.fe1))
disp('firm')
sum(isnan(fepotexp.fe2))
clear fepotexp fepotexp fpotexp

%generate reduced sample
data=data(data(:,ismember(colheaders{1},'rsample'))==1,~ismember(colheaders{1},'rsample'));
colheaders{1}=colheaders{1}(~ismember(colheaders{1},'rsample'));
%identify connected groups, keep only largest
g_id=size(data,2)+1;
i_id=find(ismember(colheaders{1},{'persnr'}));
f_id=find(ismember(colheaders{1},{'idnum'}));
i=0;
data(:,g_id)=0;
data=sortrows(data,[i_id f_id]);

while nnz(data(:,g_id))<size(data,1),
    i=i+1;
    b=zeros(max(data(:,i_id)),1);
    d=zeros(max(data(:,f_id)),1);
    g=zeros(size(data,1),1);
    alag=[];
    g(data(:,f_id)==data(find(data(:,g_id)==0,1),f_id))=i;
    a=unique(data(g==i,i_id));

while length(a)>length(alag),
    b(a)=1;
    g=(b(data(:,i_id))==1) | (g==1);
    alag=a;
    c=unique(data(g==1,f_id));
    d(c)=1;
    g=(d(data(:,f_id))==1) | (g==1);
    a=unique(data(g==1,i_id));
end
data(:,g_id)=data(:,g_id)+g.*i;
end
clear a alag b c d g i
disp('Number of observations in 20 largest connected groups')
a=histc(data(:,g_id),1:max(data(:,g_id)));
b=sortrows(a,-1);
b(1:min(length(b),20))
disp('Number of observations in remaining groups')
if length(b)>20,
	sum(b(21:end))
end

%keep only observations in largest group
[a,b]=max(a);
data=data(data(:,g_id)==b,1:end-1);
clear a b

disp('run same models as before on reduced sample')
%run match effects model with potential experience
disp('Match effects model with potential experience, reduced sample')
vnames=colheaders{1}(~ismember(colheaders{1},{'persnr' 'idnum' 'matchid' 'ltag_entg' 'tage_erw' 'tage_erw_sq' 'tage_erw_cu' 'tage_erw_qu'}));
tic
[bpotexp,fepotexp,fpotexp]=fixed(data(:,~ismember(colheaders{1},{'tage_erw' 'tage_erw_sq' 'tage_erw_cu' 'tage_erw_qu'})),4,5:4+length(vnames),1:3,'coef',1:2)
toc
disp('Output')
%arrange and save output
bpotexp.beta(:,2)=sqrt(diag(bpotexp.covb));
disp('Writing fixed effects to file, numbers below just indicate whether files were opened (fid=3), how many bytes were written and whether files were closed')
disp('individual fixed effects')
fid=fopen('p3_mat_ife_pemr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe1))
fclose(fid)
disp('firm fixed effects')
fid=fopen('p3_mat_ffe_pemr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe2))
fclose(fid)
disp('match fixed effects')
fid=fopen('p3_mat_mfe_pemr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.me))
fclose(fid)

disp('Regression Output')
disp('Statistics: # of observations, # of individuals, # of firms, # of matches, total sum of squares, residual sum of squares, R2, F-statistics')
fpotexp
disp('Coefficients and SEs')
for j=1:length(vnames),
    fprintf(1,'%s \t %12.8e \t %12.8e \n',char(vnames(j)),bpotexp.beta(j,1),bpotexp.beta(j,2));
end
disp('Covariance Matrix of the coefficient estimates')
bpotexp.covb
disp('below are just some checks whether things went right')
%get number and size of connected groups
disp('Number of people in each connected group')
histc(fepotexp.fe1(:,3),1:max(fepotexp.fe1(:,3)))
disp('Number of firms in each connected group')
histc(fepotexp.fe2(:,3),1:max(fepotexp.fe2(:,3)))
%check to see whether any effects were not calculated
disp('Missing fixed effects')
disp('individual')
sum(isnan(fepotexp.fe1))
disp('firm')
sum(isnan(fepotexp.fe2))
disp('match')
sum(isnan(fepotexp.me))
disp('# of non-zero match effects')
sum(abs(fepotexp.me(:,4))>0.000001)
clear bpotexp fepotexp fpotexp

%run two way fixed effects model with potential experience
disp('Two way fixed effects model with potential experience on reduced sample')
tic
[bpotexp,fepotexp,fpotexp]=fixed(data(:,~ismember(colheaders{1},{'tage_erw' 'tage_erw_sq' 'tage_erw_cu' 'tage_erw_qu'})),4,5:4+length(vnames),1:2,'coef',1:2)
toc
disp('Output')
%arrange and save output
disp('Writing fixed effects to file, numbers below just indicate whether files were opened (fid=3), how many bytes were written and whether files were closed')
disp('individual fixed effects')
bpotexp.beta(:,2)=sqrt(diag(bpotexp.covb));
fid=fopen('p3_mat_ife_petr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe1))
fclose(fid)
disp('firm fixed effects')
fid=fopen('p3_mat_ffe_petr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe2))
fclose(fid)

disp('Regression Output')
disp('Statistics: # of observations, # of individuals, # of firms, # of matches, total sum of squares, residual sum of squares, R2, F-statistics')
fpotexp
disp('Coefficients and SEs')
for j=1:length(vnames),
    fprintf(1,'%s \t %12.8e \t %12.8e \n',char(vnames(j)),bpotexp.beta(j,1),bpotexp.beta(j,2));
end
disp('Covariance Matrix of the coefficient estimates')
bpotexp.covb
disp('below are just some checks whether things went right')
%get number and size of connected groups
disp('Number of people in each connected group')
histc(fepotexp.fe1(:,3),1:max(fepotexp.fe1(:,3)))
disp('Number of firms in each connected group')
histc(fepotexp.fe2(:,3),1:max(fepotexp.fe2(:,3)))
%check to see whether any effects were not calculated
disp('Missing fixed effects')
disp('individual')
sum(isnan(fepotexp.fe1))
disp('firm')
sum(isnan(fepotexp.fe2))
clear fepotexp fepotexp fpotexp

%run match effects model with actual experience
disp('Match effects model with actual experience on reduced sample')
vnames=colheaders{1}(~ismember(colheaders{1},{'persnr' 'idnum' 'matchid' 'ltag_entg' 'pot_exp_sq' 'pot_exp_cu' 'pot_exp_qu'}));
tic
[bpotexp,fepotexp,fpotexp]=fixed(data(:,~ismember(colheaders{1},{'pot_exp_sq' 'pot_exp_cu' 'pot_exp_qu'})),4,5:4+length(vnames),1:3,'coef',1:2)
toc
disp('Output')
%arrange and save output
bpotexp.beta(:,2)=sqrt(diag(bpotexp.covb));
disp('Writing fixed effects to file, numbers below just indicate whether files were opened (fid=3), how many bytes were written and whether files were closed')
disp('individual fixed effects')
fid=fopen('p3_mat_ife_amr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe1))
fclose(fid)
disp('firm fixed effects')
fid=fopen('p3_mat_ffe_amr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe2))
fclose(fid)
disp('match fixed effects')
fid=fopen('p3_mat_mfe_amr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.me))
fclose(fid)

disp('Regression Output')
disp('Statistics: # of observations, # of individuals, # of firms, # of matches, total sum of squares, residual sum of squares, R2, F-statistics')
fpotexp
disp('Coefficients and SEs')
for j=1:length(vnames),
    fprintf(1,'%s \t %12.8e \t %12.8e \n',char(vnames(j)),bpotexp.beta(j,1),bpotexp.beta(j,2));
end
disp('Covariance Matrix of the coefficient estimates')
bpotexp.covb
disp('below are just some checks whether things went right')
%get number and size of connected groups
disp('Number of people in each connected group')
histc(fepotexp.fe1(:,3),1:max(fepotexp.fe1(:,3)))
disp('Number of firms in each connected group')
histc(fepotexp.fe2(:,3),1:max(fepotexp.fe2(:,3)))
%check to see whether any effects were not calculated
disp('Missing fixed effects')
disp('individual')
sum(isnan(fepotexp.fe1))
disp('firm')
sum(isnan(fepotexp.fe2))
disp('match')
sum(isnan(fepotexp.me))
disp('# of non-zero match effects')
sum(abs(fepotexp.me(:,4))>0.000001)
clear bpotexp fepotexp fpotexp

%run two way fixed effects model with actual experience
disp('Two way fixed effects model with actual experience on reduced sample')
tic
[bpotexp,fepotexp,fpotexp]=fixed(data(:,~ismember(colheaders{1},{'pot_exp_sq' 'pot_exp_cu' 'pot_exp_qu'})),4,5:4+length(vnames),1:2,'coef',1:2)
toc
disp('Output')
%arrange and save output
disp('Writing fixed effects to file, numbers below just indicate whether files were opened (fid=3), how many bytes were written and whether files were closed')
disp('individual fixed effects')
bpotexp.beta(:,2)=sqrt(diag(bpotexp.covb));
fid=fopen('p3_mat_ife_atr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe1))
fclose(fid)
disp('firm fixed effects')
fid=fopen('p3_mat_ffe_atr.csv','wt')
fprintf(fid,'%12.8e,%12.8e,%12.8e \n',transpose(fepotexp.fe2))
fclose(fid)

disp('Regression Output')
disp('Statistics: # of observations, # of individuals, # of firms, # of matches, total sum of squares, residual sum of squares, R2, F-statistics')
fpotexp
disp('Coefficients and SEs')
for j=1:length(vnames),
    fprintf(1,'%s \t %12.8e \t %12.8e \n',char(vnames(j)),bpotexp.beta(j,1),bpotexp.beta(j,2));
end
disp('Covariance Matrix of the coefficient estimates')
bpotexp.covb
disp('below are just some checks whether things went right')
%get number and size of connected groups
disp('Number of people in each connected group')
histc(fepotexp.fe1(:,3),1:max(fepotexp.fe1(:,3)))
disp('Number of firms in each connected group')
histc(fepotexp.fe2(:,3),1:max(fepotexp.fe2(:,3)))
%check to see whether any effects were not calculated
disp('Missing fixed effects')
disp('individual')
sum(isnan(fepotexp.fe1))
disp('firm')
sum(isnan(fepotexp.fe2))
clear fepotexp fepotexp fpotexp

quit

