*Program file of project fdz371 - p4_analysis.do
*Contents: perform analysis on fixed effects estimates

*This file runs the main analyses, i.e. it regresses the fixed effects estimates on the covariates created in p1_prepare.do and p2_summarize.do.
*Additionally, it analyzes the correlation of the fixed effects and plots their densities.

*Level of aggregation: For the firm fixed effects, each data line represents a firm, i.e. aggregates information on a firm over all years the firm is in the sample.
*Similarly, for the individual fixed effects, a data line represents an individual, for the match fixed effects a data line represents a unique individual-firm combination

*analyzing firm fixed effects
cap log close
log using log/p4_analysis_firms.log, text replace

*make stata wait until matlab is finished (checks for existence of file every 60s after 5 h break. Stops after 24 h if there is still no file)
capture confirm file data/p3_mat_ffe_atr.csv
disp _rc
if _rc>0 sleep 18000000
local i=300
while _rc>0 & `i'<1440 {
sleep 60000
local i=`i'+1
capture confirm file data/p3_mat_ffe_atr.csv
if _rc==0 | mod(`i',30)==0 {
disp _rc
disp `i'
}
}

set mem 1000m

*the loop just does the same analysis for all 8 models estimated in the matlab file
local i=1
foreach x in pem pet am at pemr petr amr atr {
disp "import firm fixed effects"
insheet using data/p3_mat_ffe_`x'.csv, clear comma
ren v1 idnum
ren v2 ffe_`x'
ren v3 groupid
label var idnum "Betriebs ID nach Betriebspanel"
label var ffe_`x' "Firm fixed effect from model `x'"
label var groupid "ID of the connected group"
compress
destring, replace force
drop if ffe_`x'==.
disp "create dataset containing all firm fixed effects"
if `i'>1 {
sort idnum
merge idnum using data/p4_analysis_ffe.dta, keep(idnum ffe*)
}
sort idnum
save data/p4_analysis_ffe.dta, replace
if `i'>1 {
tab _merge
keep if _merge==1 | _merge==3
}
keep idnum ffe_`x' groupid

disp "merge with firm covariates"
sort idnum
merge idnum using data/p2_summarize_firm.dta

disp "recode variables"
recode eigentum (7=9999)

disp "plot density"
graph twoway kdensity ffe_`x' [aw=hrf_quer], saving(log/p4_analysis_ffe_`x', asis replace) title("Density of Firm Fixed Effects") xtitle("Firm Fixed Effect") ytitle("Density")

disp "regressions of firm fixed effects on covariates"
char _dta[omit] "prevalent"
xi: reg ffe_`x' i.bran i.bula i.form i.eigentum i.grjahr i.single i.weiterb i.betrrat i.tarif i.work_own [pw=hrf_quer]
est store `x'
local i=`i'+1
}

*display table of coefficients and SEs from regressions above
est table pem pet am at pemr petr amr atr,se l stats(N r2 F rss)

describe
clear

cap log close

*analyzing individual fixed effects
log using log/p4_analysis_indiv.log, text replace
*the loop just does the same analysis for all 8 models estimated in the matlab file
local i=1
foreach x in pemr petr amr atr pem pet am at {
disp "import individual fixed effects"
insheet using data/p3_mat_ife_`x'.csv, clear comma
ren v1 persnr
ren v2 ife_`x'
ren v3 groupid
label var persnr "Personen ID"
label var ife_`x' "Individual fixed effect from model `x'"
label var groupid "ID of the connected group"
compress
destring, replace force
drop if ife_`x'==.
disp "create dataset containing all individual fixed effects"
sort persnr
if `i'>1 {
sort persnr
merge persnr using data/p4_analysis_ife.dta, keep(persnr ife*)
}
sort persnr
save data/p4_analysis_ife.dta, replace
if `i'>1 {
tab _merge
keep if _merge==1 | _merge==3
}
keep persnr ife_`x' groupid

disp "merge with individual covariates"
sort persnr
merge persnr using data/p2_summarize_indiv.dta
tab _merge
keep if _merge==1 | _merge==3
drop _merge

disp "plot density"
graph twoway kdensity ife_`x' [aw=hrf_quer], saving(log/p4_analysis_ife_`x', asis replace) title("Density of Individual Fixed Effects") xtitle("Individual Fixed Effect") ytitle("Density")

disp "check whether summary stats for reduced sample are correct"
tab rsample

disp "run regressions of individual fixed effects on covariates"
char _dta[omit] "prevalent"
xi: reg ife_`x' i.frau i.nation_gr i.bild i.yrentry agentry [pw=hrf_quer]
est store `x'
local i=`i'+1
}

*display table of coefficients and SEs from regressions above
est table pem pet am at pemr petr amr atr,se l stats(N r2 F rss)
describe
clear

cap log close


*analyzing match fixed effects
log using log/p4_analysis_match.log, text replace
*the loop just does the same analysis for all 4 match effects models estimated in the matlab file
local i=1
foreach x in pem am pemr amr {
disp "import individual fixed effects"
insheet using data/p3_mat_mfe_`x'.csv, clear comma
ren v1 persnr
ren v2 idnum
ren v3 matchid
ren v4 mfe_`x'
label var persnr "Personen ID"
label var idnum "Betriebs ID nach Betriebspanel"
label var matchid "Match ID"
label var mfe_`x' "Match fixed effect from model `x'"
compress
destring, replace force
drop if mfe_`x'==.
disp "create dataset containing all match fixed effects"
sort matchid
if `i'>1 {
sort matchid
merge matchid using data/p4_analysis_mfe.dta, keep(persnr idnum mfe*)
}
sort matchid
save data/p4_analysis_mfe.dta, replace
if `i'>1 {
tab _merge
keep if _merge==1 | _merge==3
}
keep persnr idnum matchid mfe_`x'

disp "merge with match covariates"
sort matchid
merge matchid using data/p2_summarize_match.dta
tab _merge
keep if _merge==1 | _merge==3
drop _merge

disp "drop people with match effects that are not identified"
bysort persnr: egen nmatch=max(n_match)
drop if nmatch<=1 | nmatch==.
drop nmatch

disp "check whether summary stats for reduced sample are correct"
tab rsample

disp "get # of movers"
codebook persnr

disp "plot density"
graph twoway kdensity mfe_`x' [aw=hrf_quer], saving(log/p4_analysis_ife_`x', asis replace) title("Density of Identified Match Fixed Effects") xtitle("Match Fixed Effect") ytitle("Density")

disp "merge with firm fixed effects"
sort idnum
merge idnum using data/p4_analysis_ffe.dta, keep(ffe_`x')
tab _merge
keep if _merge==1 | _merge==3
drop _merge

disp "generate reduced lagged spell variable"
gen estat2_8tv_s=1 if estat2_8tv==1
replace estat2_8tv_s=2 if estat2_8tv==2 | estat2_8tv==3 | estat2_8tv==4
replace estat2_8tv_s=3 if estat2_8tv==5
replace estat2_8tv_s=4 if estat2_8tv==6
replace estat2_8tv_s=. if estat2_8tv==.
tab estat2_8tv_s estat2_8tv, missing

disp "set lagged spells to zero for people first entering the labor market"
sum tage_estat2_8tv if estat2_8tv==1
replace tage_estat2_8tv=0 if estat2_8tv==1

disp "generate higher order terms of age and days in labor market"
gen age_sq=age^2
label var age_sq "alter ^2"
gen days_lm_sq=days_lm^2
label var days_lm_sq "tage seit eintritt ^2"

disp "run regressions (with different specifications) of firm and match fixed effects on match specific covariates"
char estat2_8tv[omit] 5
char estat2_8tv_s[omit] 3
disp "no interactions"
disp "full estat2_8tv"
xi: reg mfe_`x' i.parttime i.estat2_8tv*tage_estat2_8tv i.jahr tage_lst2 days_lm days_lm_sq age age_sq i.n_match [pw=hrf_quer]
capture noisily test _IestXtag_2=_IestXtag_3=_IestXtag_4
capture noisily test _Iestat2_8t_2=_Iestat2_8t_3=_Iestat2_8t_4
est store `x'_nfm
xi: reg ffe_`x' i.parttime i.estat2_8tv*tage_estat2_8tv i.jahr tage_lst2 days_lm days_lm_sq age age_sq i.n_match [pw=hrf_quer]
capture noisily test _IestXtag_2=_IestXtag_3=_IestXtag_4
capture noisily test _Iestat2_8t_2=_Iestat2_8t_3=_Iestat2_8t_4
est store `x'_nff
disp "reduced estat2_8tv"
xi: reg mfe_`x' i.parttime i.estat2_8tv_s*tage_estat2_8tv i.jahr tage_lst2 days_lm days_lm_sq age age_sq i.n_match [pw=hrf_quer]
est store `x'_nrm
xi: reg ffe_`x' i.parttime i.estat2_8tv_s*tage_estat2_8tv i.jahr tage_lst2 days_lm days_lm_sq age age_sq i.n_match [pw=hrf_quer]
est store `x'_nrf
disp "interactions with female"
disp "reduced estat2_8tv"
xi: reg mfe_`x' i.parttime i.estat2_8tv_s*tage_estat2_8tv i.jahr tage_lst2 i.frau|days_lm i.frau|days_lm_sq i.frau|age i.frau|age_sq i.frau*i.n_match [pw=hrf_quer]
capture noisily testparm _IfraXn_m_*
capture noisily testparm _IfraX*
est store `x'_yrm
xi: reg ffe_`x' i.parttime i.estat2_8tv_s*tage_estat2_8tv i.jahr tage_lst2 i.frau|days_lm i.frau|days_lm_sq i.frau|age i.frau|age_sq i.frau*i.n_match [pw=hrf_quer]
capture noisily testparm _IfraXn_m_*
capture noisily testparm _IfraX*
est store `x'_yrf
disp "full estat2_8tv"
xi: reg mfe_`x' i.parttime i.estat2_8tv*tage_estat2_8tv i.jahr tage_lst2 i.frau|days_lm i.frau|days_lm_sq i.frau|age i.frau|age_sq i.frau*i.n_match [pw=hrf_quer]
capture noisily test _IestXtag_2=_IestXtag_3=_IestXtag_4
capture noisily test _Iestat2_8t_2=_Iestat2_8t_3=_Iestat2_8t_4
capture noisily testparm _IfraXn_m_*
capture noisily testparm _IfraX*
est store `x'_yfm
xi: reg ffe_`x' i.parttime i.estat2_8tv*tage_estat2_8tv i.jahr tage_lst2 i.frau|days_lm i.frau|days_lm_sq i.frau|age i.frau|age_sq i.frau*i.n_match [pw=hrf_quer]
capture noisily test _IestXtag_2=_IestXtag_3=_IestXtag_4
capture noisily test _Iestat2_8t_2=_Iestat2_8t_3=_Iestat2_8t_4
capture noisily testparm _IfraXn_m_*
capture noisily testparm _IfraX*
est store `x'_yff
capture noisily est table `x'_nfm `x'_nff `x'_nrm `x'_nrf `x'_yfm `x'_yff `x'_yrm `x'_yrf,se stats(N r2 F rss)
local i=`i'+1
}
describe
clear
cap log close

*get correlation between fixed and firm effects
log using log/p4_analysis_zcorr.log, text replace
*create dataset with all fixed effects
use data/p4_analysis_mfe.dta, clear
cap drop _merge
sort idnum
merge idnum using data/p4_analysis_ffe.dta, keep(idnum groupid ffe*)
tab _merge
drop _merge
sort persnr
merge persnr using data/p4_analysis_ife.dta, keep(persnr ife*)
tab _merge
drop _merge
*get correlations of fixed effects from different models
*firm fixed effects
pwcorr ffe_*, sig
*individual fixed effects
pwcorr ife_*, sig
*match fixed effects
pwcorr mfe_*, sig

*correlation of firm fixed effects and individual fixed effects for each model
foreach x in pem pet am at pemr petr amr atr {
pwcorr ffe_`x' ife_`x',sig
}

clear *
cap log close
