****NOTE: Credit for writing key parts of this file go to Marin Drlje. Any remaining errors have likely been introduced by me.
****This file generates simulated data and runs different algorithms to estimate large fixed effects models according to arguments provided in the command line. The ado files for the algorithms have to be installed prior to running this file. The CL arguments are (in the following order): <# of firms> <# of individuals> <# of time periods per individual> <# of times algorithms are run> <methods to run (list in quotes)> <SD of error term (as fraction of SD of explained variation)>. For example, to generate data with 1000 firms, 100000 individuals, 10 time periods and the SD of the error term being a fifth of the SD of the explained part of y five times and run all algorithms each time: do run_simulations.do 1000 100000 10 5 "match twfe reg2hdfe a2reg felsdvreg reghdfe" 0.2
****If no arguments are provided, the program will generate a small test data set and run all algorithms. Use simulations_master.do to run it repeatedly with different setups. 


***setup
if strmatch("`1'","")==1 {
	local t=10
	local n_indiv=10000
	local n_firms=100
	local iter=3
	*to run all methods: "match twfe reg2hdfe a2reg felsdvreg reghdfe" 
	local methods "match twfe reg2hdfe a2reg felsdvreg reghdfe"
	local res_sd=0.2
}
else {
	local n_indiv=`1' 
	local n_firms=`2'
	local t=`3'
	local iter=`4'
	local methods `5'
	local res_sd=`6'
}
local noisy `7'
if strmatch("`noisy'","yes")==0 local qui qui
local n_coef=5

local n_methods=wordcount("`methods'")
mat def times=J(`iter',`n_methods',0)
*order "`methods'" such that column names are right at end (should be in order it is run)
local cnames
foreach x in match twfe reg2hdfe a2reg felsdvreg reghdfe {
	if strmatch("`methods'","*`x'*")==1 local cnames `cnames' `x' 
}

***start loop
foreach i of numlist 1/`iter' {
	di "Iteration `i'"
	qui {
		clear
		***generate data
		local N=`n_indiv'+1
		local nf=`n_firms'
		*Creating means covariates matrices of the inputs
		matrix M = 5, -6, 0.5, 3, 2
		matrix V = (9, 5, 2, 3, 4 \ 5, 9, 1, 7, 3 \ 2, 1, 9, 2 ,1 \3 ,7, 2, 9, 4\ 4, 3, 1, 4, 9)

		loc o=`N'*`t'
		set obs `o'

		drawnorm x1 x2 x3 x4 x5, n(`o') cov(V) means(M)
		gen iid=floor(_n/`t')
		drop if _n<=9
		drop if _n==_N
		local N=`N'-1
		loc o=`N'*`t'
		set obs `o'
		gen sorting=_n


		*creating person fixed effect. at the moment it depends only on the mean of variable x1
		gen ind_fe=rnormal(0,1)
		bysort iid: replace ind_fe=ind_fe[1]
		egen mean_v = mean(x1), by(iid)
		bysort iid: replace ind_fe=mean_v+ind_fe
		drop mean_v

		*creating firm draws. currently independent draws
		gen firm=floor(runiform()*`nf'+1)
		bysort iid: replace firm=firm[1]

		gen last_digit=firm-int(firm/10)*10

		gen firm2=floor(runiform()*`nf'+1)
		gen temp=runiform()
		gen firm22=floor(runiform()*`nf'/10+1)
		bysort iid: replace firm2=10*firm22+last_digit if temp>0.5
		bysort iid: replace firm2=firm2[1]

		replace last_digit=firm2-int(firm2/10)*10
		drop temp

		gen firm3=floor(runiform()*`nf'+1)
		gen temp=runiform()
		gen firm33=floor(runiform()*`nf'/10+1)
		bysort iid: replace firm3=10*firm33+last_digit if temp>0.5
		bysort iid: replace firm3=firm3[1]

		*movers
		gen tmp=(mod(_n,`t')==1)
		gen rand=runiform() if tmp==1
		gen nmove=(rand>0.8) if tmp==1
		replace nmove=2 if rand>0.96 & tmp==1

		drop rand

		gen rand=ceil(runiform()*(`t')) if tmp==1 & nmove==1
		replace rand=ceil(runiform()*(`t'-1)) if tmp==1 & nmove==2
		gen rand2=`t'-ceil(runiform()*(`t'-rand))+1 if tmp==1 & nmove==2

		bysort iid: replace firm=firm2 if _n>=rand[1]
		bysort iid: replace firm=firm3 if _n>=rand2[1]
		drop firm2 firm3 tmp rand rand2 nmove

		*creating firm fixed effect. At the moment it depends only on the mean(x1)
		gen firm_fe=rnormal(0,1)
		bysort firm:replace firm_fe=firm_fe[1]
		egen mean_v=mean(x1), by(firm)
		bysort firm: replace firm_fe=mean_v+firm_fe
		sort sorting
		drop mean_v

		gen y=x1+2*x2+3*x3+4*x4+5*x5+firm_fe+ind_fe
		qui sum y
		replace y=y+rnormal(0,`res_sd'*`r(sd)')
		rename firm fid

		keep y x1 x2 x3 x4 x5 iid fid
		compress
		
	}
	***run algoritms
	local m=1
	cap mat drop coef

	*TWFE, match effect (would run faster with replace option, but would need to re-load data)
	if strmatch("`methods'","*match*")==1 {
		sca def a=clock("$S_TIME","hms")
		`qui' twfe y x*, ids(iid fid) match 
		mat def times[`i',`m']=clock("$S_TIME","hms")-a
		if times[`i',`m']<0 mat def times[`i',`m']=times[`i',`m']+86400000
		di "TWFE match: " times[`i',`m']/1000
		mat def b=e(b)
		mat def coef=(nullmat(coef)\b[1,1..`n_coef'])
		keep y x1 x2 x3 x4 x5 iid fid
		`qui' compress
		local m=`m'+1
	}

	*TWFE, no match effect (would run faster with replace option, but would need to re-load data)
	if strmatch("`methods'","*twfe*")==1 {
		sca def a=clock("$S_TIME","hms")
		`qui' twfe y x*, ids(iid fid) 
		mat def times[`i',`m']=clock("$S_TIME","hms")-a
		if times[`i',`m']<0 mat def times[`i',`m']=times[`i',`m']+86400000
		di "TWFE: " times[`i',`m']/1000
		mat def b=e(b)
		mat def coef=(nullmat(coef)\b[1,1..`n_coef'])
		mat def d=(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])*(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])'
		local d=d[1,1]
		if `d'>0.001 di "Warning: Coefficients differ"
		keep y x1 x2 x3 x4 x5 iid fid
		`qui' compress
		local m=`m'+1
	}

	*re2hdfe
	if strmatch("`methods'","*reg2hdfe*")==1 {
		sca def a=clock("$S_TIME","hms")
		`qui' reg2hdfe y x*, id1(iid) id2(fid)
		mat def times[`i',`m']=clock("$S_TIME","hms")-a
		if times[`i',`m']<0 mat def times[`i',`m']=times[`i',`m']+86400000
		di "Reg2hdFE: " times[`i',`m']/1000
		mat def b=e(b)
		mat def coef=(nullmat(coef)\b[1,1..`n_coef'])
		mat def d=(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])*(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])'
		local d=d[1,1]
		if `d'>0.001 di "Warning: Coefficients differ"
		local m=`m'+1
	}

	*a2reg
	if strmatch("`methods'","*a2reg*")==1 {
		sca def a=clock("$S_TIME","hms")
		`qui' a2reg y x*, individual(iid) unit(fid)
		mat def times[`i',`m']=clock("$S_TIME","hms")-a
		if times[`i',`m']<0 mat def times[`i',`m']=times[`i',`m']+86400000
		di "a2reg: " times[`i',`m']/1000
		mat def b=e(b)
		mat def coef=(nullmat(coef)\b[1,1..`n_coef'])
		mat def d=(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])*(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])'
		local d=d[1,1]
		if `d'>0.001 di "Warning: Coefficients differ"
		local m=`m'+1
	}
	
	* felsdvreg
	if strmatch("`methods'","*felsdvreg*")==1 {
		sca def a=clock("$S_TIME","hms")
		`qui' felsdvreg y x*, ivar(iid) jvar(fid) peff(iid1) feff(fid1) xb(name1) res(name2) mover(name3) group(name4) mnum(name5) pobs(name6) 
		mat def times[`i',`m']=clock("$S_TIME","hms")-a
		if times[`i',`m']<0 mat def times[`i',`m']=times[`i',`m']+86400000
		di "FeLsDvReg: " times[`i',`m']/1000
		mat def b=e(b)
		mat def coef=(nullmat(coef)\b[1,1..`n_coef'])
		mat def d=(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])*(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])'
		local d=d[1,1]
		if `d'>0.001 di "Warning: Coefficients differ"
		keep y x1 x2 x3 x4 x5 iid fid
		`qui' compress
		local m=`m'+1
	}

	*reghdfe
	if strmatch("`methods'","*reghdfe*")==1 {
		sca def a=clock("$S_TIME","hms")
		`qui' capture reghdfe y x*, absorb(iid fid)
		mat def times[`i',`m']=clock("$S_TIME","hms")-a
		if times[`i',`m']<0 mat def times[`i',`m']=times[`i',`m']+86400000
		di "reghdfe: " times[`i',`m']/1000
		mat def b=e(b)
		mat def coef=(nullmat(coef)\b[1,1..`n_coef'])
		mat def d=(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])*(coef[`m',1..`n_coef']-coef[1,1..`n_coef'])'
		local d=d[1,1]
		if `d'>0.001 di "Warning: Coefficients differ"
		local m=`m'+1
	}	

	mat rownames coef=`cnames'
	mat list coef
}

***export results
clear
mat def times=(times\J(1,`iter',1)*times/`iter')
mat list times
mat colnames times=`cnames'
svmat times, names(col)
gen run_number=_n
tostring run_number, replace
replace run_number="Average" if _n==_N
export excel run_number `methods' using times.xlsx, sh(`n_indiv'_`n_firms') sheetreplace firstrow(var) nol
